@@ -15,7 +15,7 @@
 ** individual tokens and sends those tokens one-by-one over to the
 ** parser for analysis.
 **
-** $Id: tokenize.c,v 1.1.1.1 2002/02/18 17:39:01 matt Exp $
+** $Id: tokenize.c,v 1.2 2002/02/20 13:34:53 matt Exp $
 */
 #include "sqliteInt.h"
 #include "os.h"
@@ -346,6 +346,9 @@ static int sqliteGetToken(const unsigned char *z, int *tokenType){
   return 1;
 }
 
+#undef malloc
+#undef free
+
 /*
 ** Run the parser on the given SQL string.  The parser structure is
 ** passed in.  An SQLITE_ status code is returned.  If an error occurs
